if not Blessed then return end
local mod = Blessed
local players, playerList = {}, {}

local priorities = {
	["ROGUE"] = {"gkings", "gmight", "gsanct"},
	["WARRIOR"] = {"gkings", "gmight", "gsanct"},
	["DEATHKNIGHT"] = {"gkings", "gmight", "gsanct"},
	["PRIEST"] = {"gkings", "gwisdom", "gsanct"},
	["WARLOCK"] = {"gkings", "gwisdom", "gsanct"},
	["MAGE"] = {"gkings", "gwisdom", "gsanct"},
	["HUNTER"] = {"gkings", "gmight", "gwisdom", "gsanct"},
	["PALADIN"] = {
		{"gkings", "gwisdom", "gmight", "gsanct"},		-- Holy
		{"gsanct", "gkings", "gwisdom", "gmight"},		-- Prot
		{"gkings", "gmight", "gwisdom", "gsanct"},		-- Ret
	},
	["DRUID"] = {
		{"gkings", "gwisdom", "gmight", "gsanct"},		-- Balance
		{"gkings", "gmight", "gwisdom", "gsanct"},		-- Feral
		{"gkings", "gwisdom", "gmight", "gsanct"},		-- Resto
	},
	["SHAMAN"] = {
		{"gkings", "gwisdom", "gmight", "gsanct"},		-- Elemental
		{"gkings", "gmight", "gwisdom", "gsanct"},		-- Enhance
		{"gkings", "gwisdom", "gmight", "gsanct"},		-- Resto
	}
}

local auras = {
	conc = 19746,
	devo = 48942,
	fire = 48947,
	frost = 48945,
	ret = 54043,
	shadow = 48943
}

do
	local talents = {
		1, 9, 		-- Conc aura
		1, 10, 		-- Wisdom
		2, 7, 		-- Sanc
		2, 11, 		-- Devo aura
		3, 5, 		-- Might
		3, 14		-- Ret aura
	}
	local buffer = {}
	function Blessed:SendTalents()
		wipe(buffer)
		for tree = 1, #talents, 2 do
			local slot = talents[tree+1]
			local _, _, _, _, points, maxPoints = GetTalentInfo(tree, slot)
			tinsert(buffer, math.floor((points * 10) / maxPoints))
		end
		local talentString = table.concat(buffer, ":")
		local msg = ("T:%s"):format(talentString)
		self:SendCommMessage(self.commPrefix, msg, self:GetDistribution())
		
		-- Store our data in our local table, too.
		if not players[UnitName("player")] then
			self:OnCommReceived(nil, msg, nil, UnitName("player"))
		end
	end
end

function mod:WipeCachedBlessers()
	wipe(players)
	wipe(playerList)
end

function mod:OnCommReceived(prefix, data, distribution, sender)
	local cmd, rest = strsplit(":", data, 2)
	
	-- When we receive another paladin's talents, store them and then trigger buff reassignment.
	if cmd == "T" then
		local conc, wis, sanc, devo, might, ret = strsplit(":", rest)
		if not players[sender] then
			players[sender] = {}
			tinsert(playerList, sender)
		end
		players[sender].conc 	= tonumber(conc)
		players[sender].wisdom 	= tonumber(wis)
		players[sender].sanc 	= tonumber(sanc)
		players[sender].devo 	= tonumber(devo)
		players[sender].might 	= tonumber(might)
		players[sender].ret 	= tonumber(ret)
		
		self:AssignBuffs()
	end
end

local assignments = {AURA = {}, BLESSING = {}}
local assignAuras, assignGreaters, assignLessers, assignAll
local auras = {"conc", "devo", "ret", "shadow", "frost", "fire"}
do
	
	local weights = {}
	local used, usedAuras = {}, {}	
	function assignForSet(set)		
		wipe(weights)
		
		-- Get each player's total weight. The lower the weight, the higher priority they'll have on their max-rank aura.
		for _, aura in ipairs(set) do
			weights[player] = weights[player] + v[aura]
		end

		-- If we only have one player, then we need them to just use their highest-
		for index, aura in ipairs(set) do
			-- If we're at the last player, this player should just take their highest-rank aura that we haven't already used.
			if index == #playerList then
				local max, maxAura = -1, nil
				local p = players[playerList[#playerList]]
				for _, bestAura in ipairs(set) do
					if not usedAuras[bestAura] then
						if p[bestAura] > max then
							max = p[bestAura]
							maxAura = bestAura
						end
					end
				end
				if maxAura then
					usedAuras[maxAura] = true
					used[playerList[#playerList]] = true
					assignments.AURA[assignPlayer] = maxAura
				end
			else -- find the lowest-weight player with the highest improvement level
				local assignPlayer = nil
				local impPoints = -1
				local lowestWeight = 999
				for k, v in pairs(players) do
					if not used[k] then
						if v[aura] > impPoints and weights[k] < lowestWeight then
							impPoints = v[aura]
							lowestWeight = weights[k]
							assignPlayer = k
							used[assignPlayer] = true
							usedAuras[aura] = true
						end
					end
				end				
				if assignPlayer then
					assignments.AURA[assignPlayer] = auras[aura]
				end
			end
		end
	end
end

local function assignAll()
	wipe(assignments.AURA)
	wipe(assignments.BLESSING)
	assignAuras()
	assignGreaters()
	assignLessers()
end

function mod:AssignBuffs()
end